@@ -11,15 +11,13 @@ Original code Copyright 2009 [Waylan Limberg](http://achinghead.com)
 
 All changes Copyright 2008-2014 The Python Markdown Project
 
-License: [BSD](http://www.opensource.org/licenses/bsd-license.php)
+License: [BSD](https://opensource.org/licenses/bsd-license.php)
 
 """
 
-from __future__ import absolute_import
-from __future__ import unicode_literals
 from . import Extension
 from ..blockprocessors import BlockProcessor
-from ..util import etree
+import xml.etree.ElementTree as etree
 import re
 PIPE_NONE = 0
 PIPE_LEFT = 1
@@ -35,7 +33,7 @@ class TableProcessor(BlockProcessor):
     def __init__(self, parser):
         self.border = False
         self.separator = ''
-        super(TableProcessor, self).__init__(parser)
+        super().__init__(parser)
 
     def test(self, parent, block):
         """
@@ -202,7 +200,7 @@ class TableProcessor(BlockProcessor):
             if not throw_out:
                 good_pipes.append(pipe)
 
-        # Split row according to table delimeters.
+        # Split row according to table delimiters.
         pos = 0
         for pipe in good_pipes:
             elements.append(row[pos:pipe])
